require 'digest'

module PackagecloudDebtools
  class Checksum
    attr_accessor :type, :hash, :size, :filename, :dirname, :fullpath

    def self.parse(line, path)
      if line =~ /^([a-z0-9A-Z]+)[\t\s]+([0-9]+)[\t\s]+(.*)$/
        type = "unknown"
        case $1.length
        when 32
          type = "md5"
        when 40
          type = "sha1"
        when 64
          type = "sha256"
        else
          puts "hash length: #{$1.length}"
        end
        Checksum.new(type, $1, $2, $3, path)
      else
        raise "#{line.strip} doesnt look like a checksum!"
      end
    end

    def initialize(type, hash, size, filename, path)
      @type = type
      @hash = hash
      @size = size.to_i
      @filename = filename
      @dirname = path
      @fullpath = ::File.join(@dirname, @filename)
    end

    def verify
      calculated_file_sz = ::File.size(@fullpath)
      calculated_digest = case type
                          when "sha1"
                            Digest::SHA1.file(@fullpath).hexdigest
                          when "sha256"
                            Digest::SHA256.file(@fullpath).hexdigest
                          when "md5"
                            Digest::MD5.file(@fullpath).hexdigest
                          end

      if calculated_file_sz != @size
        raise "File size mismatch. Got: #{calculated_file_sz}, Expected: #{@size}"
      elsif calculated_digest != @hash
        raise "Hash #{@type} mistmatch. Got #{calculated_digest}, Expected: #{@hash}"
      else
        true
      end
    end
  end
end
