module PackagecloudDebtools
  class Parser
    BINARY_RELATIONSHIP_FIELDS = [ "replaces", "provides", "depends", "conflicts",
                                   "recommends", "suggests", "breaks", "pre-depends",
                                   "enhances",  "built-using" ]

    SOURCE_RELATIONSHIP_FIELDS = [ "build-depends",  "build-depends-indep",
                                   "build-conflicts", "build-conflicts-indep"]

    RELATIONSHIP_FIELDS = BINARY_RELATIONSHIP_FIELDS + SOURCE_RELATIONSHIP_FIELDS

    def self.parse_from_file(filename)
      f = ::File.new(filename)
      filetype = ::File.extname(filename)
      self.parse(f, filetype)
    end

    def self.parse(io, filetype)
      result = nil
      fieldname = nil

      h = Hash.new do |hash, key|
        if key == 'relationships'
          hash[key] = Hash.new
        end
      end

      if filetype == ".changes"
        result = Changes.new(io, h)
      elsif filetype == ".dsc"
        result = Dsc.new(io, h)
      elsif filetype == ".deb"
        result = Deb.new(io, h)
      else
        raise "Unknown filetype: #{filetype}"
      end

      result.fields['size'] = io.stat.size

      result.control_file.each_line do |line|
        if line =~ /^[\t\s]+.*/
          # if the line starts with a white space, it is part of an existing
          # field
          line = line.strip if fieldname != 'description'
          result.add_to_current_section(line, fieldname)
        else
          # if it doesnt start with a white space, it should have a ':' in
          # it
          if line.count(':') == 0 
            raise "Couldn't understand this line: #{line}"
          end

          # split the string into two chunks on the first ':'
          parts = line.split(':', 2)

          # if the second chunk is "", then parts[0] is a new multi-line
          # field
          if parts[1].strip == ""
            fieldname = parts[0].downcase.strip
            result.fields[fieldname] = Array.new
          else
            # if parts[1] is not an empty string, then this is a single line
            # field
            fieldname = parts[0].downcase.strip
            if RELATIONSHIP_FIELDS.include?(fieldname)
              result.fields['relationships'][fieldname] = parts[1].strip
            elsif fieldname != 'vcs-browser' && fieldname =~ /^vcs-(.*)/
              result.fields['vcs'] = $1
              result.fields['vcs-url'] = parts[1].strip
            elsif fieldname == 'description'
              result.fields['description'] = Array.new
              result.add_to_current_section(parts[1].lstrip, fieldname)
            else
              result.fields[fieldname] = parts[1].strip
            end
          end
        end
      end

      # only DEBs have a description hash, the others dont
      if filetype == ".deb"
        result.fix_description_and_compute_hash
      end

      result
    end
  end
end
